using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using WorkFlowCheck.Common.DTO;

namespace WorkFlowCheck.MAUI.Services.Interfaces
{
    public interface ICheckListService
    {
        Task<ApiResponseDTO<CheckListHeaderDTO>> StartNew(CheckListHeaderNewDTO checkListHeaderNewDTO);
        Task<ApiResponseDTO<CheckPointDTO>> GetCheckPoint(int checkPointId);
        Task<ApiResponseDTO<CheckPointDTO>> GetCheckPoint(string checkPointCode);


        Task<List<CheckListHeaderDTO>> GetCheckLists(int userId);
        Task<List<CheckListTemplateHeaderDTO>> GetCheckListTemplates(int userId);
        Task<List<CheckListRowDTO>> GetCheckListRowsByCheckPointCode(int userId, int checkListHeaderId, string checkPointCode);
        Task<CheckListRowDTO> UpdateCheckListRow(CheckListRowDTO checkListRowDTO);
    }
}
